/**
 * This file is part of the dcrypt project.
 *
 * Copyright: Copyright (C) dcrypt contributors 2008. All rights reserved.
 * License:   MIT
 * Authors:   Thomas Dixon
 */

module dcrypt.crypto.MAC;

public import dcrypt.crypto.params.CipherParameters;
public import dcrypt.crypto.errors.InvalidParameterError;
import dcrypt.misc.Util;

/** Base MAC class */
abstract class MAC {
    /**
     * Initialize a MAC.
     * 
     * Params:
     *     params  = Parameters to be passed to the MAC. (Key, etc.)
     */
    void init(CipherParameters params);
    
    /**
     * Introduce data into the MAC.
     * 
     * Params:
     *     input_ = Data to be processed.
     */
    void update(void[] input_);
    
    /** Returns: The name of this MAC. */
    char[] name();
    
    /** Reset MAC to its state immediately subsequent the last init. */
    void reset();
    
    /** Returns: The block size in bytes that this MAC will operate on. */
    uint blockSize();
    
    /** Returns: The output size of the MAC in bytes. */
    uint macSize();
    
    /** Returns: The computed MAC. */
    ubyte[] digest();
    
    /** Returns: The computed MAC in hexadecimal. */
    char[] hexDigest() {
        return Util.ubytesToHex(digest());
    }
}